/*
 * Java port of Bullet (c) 2008 Martin Dvorak <jezek2@advel.cz>
 *
 * Stan Melax Convex Hull Computation
 * Copyright (c) 2008 Stan Melax http://www.melax.com/
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose, 
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

package com.bulletphysics.linearmath.convexhull;

import com.bulletphysics.linearmath.MiscUtil;
import java.util.ArrayList;
import java.util.List;
import javax.vecmath.Vector3f;

/**
 *
 * @author jezek2
 */
public class ConvexH {
	
	public final List<Vector3f> vertices = new ArrayList<Vector3f>();
	public final List<HalfEdge> edges = new ArrayList<HalfEdge>();
	public final List<Plane> facets = new ArrayList<Plane>();

	public ConvexH() {
	}
	
	public ConvexH(int vertices_size, int edges_size, int facets_size) {
		MiscUtil.resize(vertices, vertices_size, Vector3f.class);
		MiscUtil.resize(edges, edges_size, HalfEdge.class);
		MiscUtil.resize(facets, facets_size, Plane.class);
	}
	
	////////////////////////////////////////////////////////////////////////////

	public static class HalfEdge {
		public short ea; // the other half of the edge (index into edges list)
		public byte v;   // the vertex at the start of this edge (index into vertices list)
		public byte p;   // the facet on which this edge lies (index into facets list)

		public HalfEdge() {
		}

		public HalfEdge(short _ea, byte _v, byte _p) {
			ea = _ea;
			v = _v;
			p = _p;
		}
	}
	
}
